LPARAM lcType,lcPath
* lcType - Distribution type
*   N - Normal; EXE binary, without debug info
*   D - Debug;, EXE binary, with debug info
*   S - Source; Source code (programs, free tables ), project and additional programs
*   H - HTML documentation
* lcPath - Output folder

LOCAL lcMainPath
IF EMPTY(lcType) OR EMPTY(lcPath)
   RETURN
ENDIF

CLEAR

DECLARE INTEGER RemoveDirectory IN kernel32 String @lpszPath
DECLARE INTEGER DeleteFile      IN kernel32 String @lpszFile
DECLARE INTEGER CopyFile IN KERNEL32 STRING @lpExistingFileName, STRING @lpNewFileName, INTEGER
DECLARE INTEGER SetLastError IN kernel32 Integer fdwError
DECLARE INTEGER GetDriveType IN kernel32 String lpszRootPathName



lcPath=AddBackSlashInPath(ALLT(lcPath))

* Pokud neexistuje  clov cesta, pak ji zalo
IF !DIRECTORY(lcPath)
   =Mkd(lcPath)
ELSE
   * jinak ji vyisti
   =RemoveDirectoryAll(lcPath,.F.)
ENDIF

lcMainPath=GetMainPath()
DELE FILE (lcMainPath+"dbfs\*.bak")
DELE FILE (lcMainPath+"dbfs\*.tbk")
DELE FILE (lcMainPath+"plug-in\*.bak")
DELE FILE (lcMainPath+"prgs\*.bak")


* Podle typu se rozhodhi
* N - Normal, bez zdrojovch kdu a debug informac
* D - Debug, bez zdrojovch kdu
* S - Source, zdrojov kdy + projekt bez EXE
* H - HTML dokumentace

* F1 - pdm.fll source
* F2 - string.fll source


@ 1,0 SAY "Distribution: "+lcType
@ 2,0 SAY "Output folder: "+lcPath


DO CASE
   CASE lcType="N"
        =CNormal(lcPath)

   CASE lcType="D"
        =CDebug(lcPath)

   CASE lcType="S"
        =CSource(lcPath)

   CASE lcType="H"
        =CHTML(lcPath)

   CASE lcType="F1"
        =CPDMFLL(lcPath)

   CASE lcType="F2"
        =CSTRINGFLL(lcPath)

ENDCASE

******************************************************************************
* Normln distribuce
******************************************************************************
PROCEDURE CNormal(lcPath)
* lcPath - Clov cesta

* pdm.exe bez debug info
* pdm.fll
* string.fll
* lang/.
* gui/.
* plug-in/.
* res/.
* test/.

LOCAL lcAlias,lcMainPath
lcAlias=SYS(2015)
lcMainPath=GetMainPath()

=WriteText("Build PDM project...",0)

USE pdm.pjx IN 0 ALIAS (lcAlias)
SELE (lcAlias)
LOCATE FOR TYPE="H"
REPLACE DEBUG WITH .F. NEXT 1
USE
KEYB "{A}" PLAIN
BUILD EXE (lcMainPath+"pdm.exe") FROM (lcMainPath+"pdm.pjx")

=WriteText("Build Loader project...",0)
USE loader\loader.pjx IN 0 ALIAS (lcAlias)
SELE (lcAlias)
LOCATE FOR TYPE="H"
REPLACE DEBUG WITH .F. NEXT 1
USE
KEYB "{A}" PLAIN
BUILD EXE (lcMainPath+"loader.exe") FROM (lcMainPath+"loader\loader.pjx")

=WriteText("Create Root PDM...",0)
=CopyFile((lcMainPath+"pdm.exe"+CHR(0)),(lcPath+"pdm.exe"+CHR(0)),0)
=CopyFile((lcMainPath+"loader.exe"+CHR(0)),(lcPath+"loader.exe"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.fll"+CHR(0)),(lcPath+"pdm.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.ini"+CHR(0)),(lcPath+"pdm.ini"+CHR(0)),0)
=CopyFile((lcMainPath+"config.fpw"+CHR(0)),(lcPath+"config.fpw"+CHR(0)),0)
=CopyFile((lcMainPath+"string.fll"+CHR(0)),(lcPath+"string.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"license.txt"+CHR(0)),(lcPath+"license.txt"+CHR(0)),0)

* Zpakuj tabulky
=WriteText("Pack Tables...",0)
=PackTables(lcMainPath+"lang",.T.)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.bak")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.tbk")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.err")


=WriteText("Copy subfolder...",0)
=CopyDirectory(lcMainPath+"lang",lcPath+"lang",.T.,.T.)
=CopyDirectory(lcMainPath+"res",lcPath+"res",.T.,.T.)
=CopyDirectory(lcMainPath+"test",lcPath+"test",.T.,.T.)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcPath+"lang,"+lcPath+"dbfs",.T.,"*.prg")
=DeleteFiles(lcPath+"test,",.T.,"*.fxp")


=CopyPlugins("N",lcPath)
=CopyGUI("N",lcPath)


******************************************************************************
* Debug distribuce
******************************************************************************
PROCEDURE CDebug(lcPath)
* lcPath - Clov cesta

* pdm.exe s debug info
* pdm.fll
* string.fll
* lang/.
* gui/.
* plug-in/.
* res/.
* test/.

LOCAL lcAlias,lcMainPath
lcAlias=SYS(2015)
lcMainPath=GetMainPath()

=WriteText("Build PDM project...",0)
USE pdm.pjx IN 0 ALIAS (lcAlias)
SELE (lcAlias)
LOCATE FOR TYPE="H"
REPLACE DEBUG WITH .T. NEXT 1
USE
KEYB "{A}" PLAIN
BUILD EXE (lcMainPath+"pdm.exe") FROM (lcMainPath+"pdm.pjx")

=WriteText("Build Loader project...",0)
USE loader\loader.pjx IN 0 ALIAS (lcAlias)
SELE (lcAlias)
LOCATE FOR TYPE="H"
REPLACE DEBUG WITH .T. NEXT 1
USE
KEYB "{A}" PLAIN
BUILD EXE (lcMainPath+"loader.exe") FROM (lcMainPath+"loader\loader.pjx")

=WriteText("Create Root PDM...",0)
=CopyFile((lcMainPath+"pdm.exe"+CHR(0)),(lcPath+"pdm.exe"+CHR(0)),0)
=CopyFile((lcMainPath+"loader.exe"+CHR(0)),(lcPath+"loader.exe"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.fll"+CHR(0)),(lcPath+"pdm.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"config.fpw"+CHR(0)),(lcPath+"config.fpw"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.ini"+CHR(0)),(lcPath+"pdm.ini"+CHR(0)),0)
=CopyFile((lcMainPath+"string.fll"+CHR(0)),(lcPath+"string.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"license.txt"+CHR(0)),(lcPath+"license.txt"+CHR(0)),0)

* Zpakuj tabulky
=WriteText("Pack Tables...",0)
=PackTables(lcMainPath+"lang",.T.)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.bak")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.tbk")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.err")

=WriteText("Copy subfolder...",0)
=CopyDirectory(lcMainPath+"lang",lcPath+"lang",.T.,.T.)
=CopyDirectory(lcMainPath+"res",lcPath+"res",.T.,.T.)
=CopyDirectory(lcMainPath+"test",lcPath+"test",.T.,.T.)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcPath+"test,",.T.,"*.fxp")

=CopyPlugins("D",lcPath)
=CopyGUI("D",lcPath)



******************************************************************************
* Source distribuce
******************************************************************************
PROCEDURE CSource(lcPath)
* lcPath - Clov cesta

* pdm.pjx/pdm.pjt
* pdm.fll
* string.fll
* vcxs/.
* lang/.
* plug-in/.
* res/.
* gui/.
* test/.
* dbfs/.
* prgs/.
* doc/.

LOCAL lcMainPath
lcMainPath=GetMainPath()


=WriteText("Create Root PDM...",0)
=CopyFile((lcMainPath+"pdm.pjx"+CHR(0)),(lcPath+"pdm.pjx"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.pjt"+CHR(0)),(lcPath+"pdm.pjt"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.fll"+CHR(0)),(lcPath+"pdm.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"pdm.ini"+CHR(0)),(lcPath+"pdm.ini"+CHR(0)),0)
=CopyFile((lcMainPath+"config.fpw"+CHR(0)),(lcPath+"config.fpw"+CHR(0)),0)
=CopyFile((lcMainPath+"string.fll"+CHR(0)),(lcPath+"string.fll"+CHR(0)),0)
=CopyFile((lcMainPath+"comments.txt"+CHR(0)),(lcPath+"comments.txt"+CHR(0)),0)
=CopyFile((lcMainPath+"synstring.prg"+CHR(0)),(lcPath+"synstring.prg"+CHR(0)),0)
=CopyFile((lcMainPath+"changestring.prg"+CHR(0)),(lcPath+"changestring.prg"+CHR(0)),0)
=CopyFile((lcMainPath+"cdistr.prg"+CHR(0)),(lcPath+"cdistr.prg"+CHR(0)),0)
=CopyFile((lcMainPath+"license.txt"+CHR(0)),(lcPath+"license.txt"+CHR(0)),0)
=CopyFile((lcMainPath+"setsenv.prg"+CHR(0)),(lcPath+"setsenv.prg"+CHR(0)),0)
=CopyFile((lcMainPath+"BuildNotes.txt"+CHR(0)),(lcPath+"BuildNotes.txt"+CHR(0)),0)
=CopyFile((lcMainPath+"RunFromSourceCode.txt"+CHR(0)),(lcPath+"RunFromSourceCode.txt"+CHR(0)),0)


* Zpakuj tabulky
=WriteText("Pack Tables...",0)
=PackTables(lcMainPath+"dbfs",.T.)
=PackTables(lcMainPath+"lang",.T.)

=CopyPlugins("S",lcPath)
=CopyGUI("S",lcPath)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,"+lcMainPath+"loader,",.T.,"*.bak")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.tbk")
=DeleteFiles(lcMainPath+"lang,"+lcMainPath+"test,"+lcMainPath+"dbfs,",.T.,"*.err")

=WriteText("Copy subfolder...",0)
=CopyDirectory(lcMainPath+"lang",lcPath+"lang",.T.,.T.)
=CopyDirectory(lcMainPath+"res",lcPath+"res",.T.,.T.)
=CopyDirectory(lcMainPath+"test",lcPath+"test",.T.,.T.)
=CopyDirectory(lcMainPath+"dbfs",lcPath+"dbfs",.T.,.T.)
=CopyDirectory(lcMainPath+"prgs",lcPath+"prgs",.T.,.T.)
=CopyDirectory(lcMainPath+"loader",lcPath+"loader",.T.,.T.)
=CopyDirectory(lcMainPath+"cplugin",lcPath+"cplugin",.T.,.T.)

=WriteText("Delete useless files...",0)
=DeleteFiles(lcPath+"loader,"+lcPath+"prgs,"+lcPath+"lang,"+lcPath+"test,"+lcPath+"dbfs,"+lcPath+"cplugin",.T.,"*.fxp")

=CopyFile((lcMainPath+"Readme.htm"+CHR(0)),(lcPath+"Readme.htm"+CHR(0)),0)
=CopyDirectory(lcMainPath+"doc",lcPath+"doc",.T.,.T.)


******************************************************************************
* Dokumentace
******************************************************************************
PROCEDURE CHTML(lcPath)
* lcPath - Clov cesta

* doc/.

LOCAL lcMainPath
lcMainPath=GetMainPath()
=CopyFile((lcMainPath+"Readme.htm"+CHR(0)),(lcPath+"Readme.htm"+CHR(0)),0)
=CopyDirectory(lcMainPath+"doc",lcPath+"doc",.T.,.T.)

******************************************************************************
* pdm.fll - source code
******************************************************************************
PROCEDURE CPDMFLL(lcPath)
LOCAL lcMainPath
lcMainPath="d:\x_disc\project\pdm\pdm0100\"
=CopyFile((lcMainPath+"Pdm0100.cpp"+CHR(0)),(lcPath+"Pdm0100.cpp"+CHR(0)),0)
=CopyFile((lcMainPath+"Pdm0100.dsp"+CHR(0)),(lcPath+"Pdm0100.dsp"+CHR(0)),0)
=CopyFile((lcMainPath+"Pdm0100.rc"+CHR(0)),(lcPath+"Pdm0100.rc"+CHR(0)),0)
=CopyFile((lcMainPath+"resource.h"+CHR(0)),(lcPath+"resource.h"+CHR(0)),0)

******************************************************************************
* string.fll - source code
******************************************************************************
PROCEDURE CSTRINGFLL(lcPath)
LOCAL lcMainPath
lcMainPath="d:\x_disc\project\pdm\string0100\"
=CopyFile((lcMainPath+"string0100.cpp"+CHR(0)),(lcPath+"string0100.cpp"+CHR(0)),0)
=CopyFile((lcMainPath+"string0100.dsp"+CHR(0)),(lcPath+"string0100.dsp"+CHR(0)),0)
=CopyFile((lcMainPath+"string0100.rc"+CHR(0)),(lcPath+"string0100.rc"+CHR(0)),0)
=CopyFile((lcMainPath+"resource.h"+CHR(0)),(lcPath+"resource.h"+CHR(0)),0)


******************************************************************************
* GUI
******************************************************************************
PROCEDURE CopyGUI(lcType,lcPath) && 
* lcType - DistributionType
* lcPath - Source path

LOCAL lcMainPath,lcFolder
lcMainPath=GetMainPath()
lcFolder="gui"

=WriteText("Generate GUI...",0)

* Pekompiluj VCX knihovny
=WriteText("Compile VCX libraries...",1)
=CompileVCXs(lcMainPath+lcFolder,.T.)

* Zpakuj tabulky
=WriteText("Pack Tables...",1)
=PackTables(lcMainPath+lcFolder,.T.)

* Pekompiluj PRG soubory
=WriteText("Compile programs...",1)
=CompilePRGs(lcMainPath+lcFolder,.T.)

* Sma nepotebn soubory
=WriteText("Delete useless files...",1)
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.bak")
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.tbk")
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.err")


* Ve pekopruj
=WriteText("Copy subfolder...",0)
=CopyDirectory(lcMainPath+lcFolder,lcPath+"gui",.F.,.T.)
=CopyDirectory(lcMainPath+lcFolder+"\img",lcPath+lcFolder+"\img",.T.,.T.)
=CopyDirectory(lcMainPath+lcFolder+"\lang",lcPath+lcFolder+"\lang",.T.,.T.)

DO CASE
   CASE lcType="N" && EXE without Debug info
        * vyma pole methods vech 
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.h")
        =DeleteFiles(lcPath+lcFolder,.T.,"*.prg")

        =WriteText("Clear VCX libraries...",1)
        =ClearVCXs(lcPath+lcFolder,.T.)

   CASE lcType="D" && EXE with Debug info
        * sma h soubory
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.h")

   CASE lcType="S" && Source Code
        * ve zkoprovno
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.fxp")

ENDCASE

ENDPROC


******************************************************************************
* Plug-ins
******************************************************************************
PROCEDURE CopyPlugIns(lcType,lcPath) && 
* lcType - DistributionType
* lcPath - Source path

LOCAL lcMainPath,lcFolder
lcMainPath=GetMainPath()
lcFolder="plug-in"

=WriteText("Generate plug-ins...",0)

* Pekompiluj VCX knihovny
=WriteText("Compile VCX libraries...",1)
=CompileVCXs(lcMainPath+lcFolder,.T.)

* Zpakuj tabulky
=WriteText("Pack Tables...",1)
=PackTables(lcMainPath+lcFolder,.T.)

* Pekompiluj PRG soubory
=WriteText("Compile programs...",1)
=CompilePRGs(lcMainPath+lcFolder,.T.)

* Sma nepotebn soubory
=WriteText("Delete useless files...",1)
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.bak")
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.tbk")
=DeleteFiles(lcMainPath+lcFolder,.T.,"*.err")



* Ve pekopruj
=WriteText("Copy subfolder...",0)
=CopyDirectory(lcMainPath+lcFolder,lcPath+lcFolder,.T.,.T.)

=DeleteFiles(lcPath+lcFolder,.T.,"ac_stats.*")
=RemoveDirectoryAll(lcPath+lcFolder+"\ac_stats",.F.)




DO CASE
   CASE lcType="N" && EXE without Debug info
        * vyma pole methods vech 
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.h")
        =DeleteFiles(lcPath+lcFolder,.T.,"*.prg")

        =WriteText("Clear VCX libraries...",1)
        =ClearVCXs(lcPath+lcFolder,.T.)

   CASE lcType="D" && EXE with Debug info
        * sma h soubory
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.h")

   CASE lcType="S" && Source Code
        * ve zkoprovno
        =WriteText("Delete useless files...",1)
        =DeleteFiles(lcPath+lcFolder,.T.,"*.fxp")

ENDCASE

ENDPROC


******************************************************************************
* Vrt cestu, kde se nachz program
******************************************************************************
PROCEDURE GetMainPath()
LOCAL lcMainPath
*lcMainPath=SYS(16,PROGRAM(-1)-2)
lcMainPath=SYS(16,1)
RETURN IIF(RAT("\",lcMainPath)>0,LEFT(lcMainPath,RAT("\",lcMainPath)),lcMainPath)
ENDPROC


******************************************************************************
* Pid backslash k cest, pokud nen
******************************************************************************
PROCEDURE AddBackSlashInPath(lcPath)
* lcPath - cesta
RETURN IIF(EMPTY(lcPath),lcPath,lcPath+IIF(RIGHT(lcPath,1)#"\","\",""))
ENDPROC



******************************************************************************
* Zkopruje obsah adrese do jinho adrese
******************************************************************************
PROCEDURE CopyDirectory(lcSource,lcDest,llChild,llLower,lcPatern)
* lcSource - Zdrojov adres
* lcDest   - Clov adres
* llChild  - Pznak, koprovn podadres
* llLower  - Pznak, zda se maj nzvy adres a soubor pevst na mal psmena
* lcPatern - Patern pro vbr soubor

IF PCOUNT()<=2 && pokud nen tet parametr
   llChild=.T. && pak nastav pznak koprovn Child Adres
ENDIF
IF PCOUNT()<=3 && 
   llLower=.F. && 
ENDIF
IF PCOUNT()<=4 && 
   lcPatern="*.*"
ENDIF
LOCAL lcSourceN,lcDestN,lic,lierr,lierrx,lii,loDir,lcFSource,lcFDest
LOCAL ARRAY laDir(1)

lcSource=ALLT("&lcSource") && preventivn odstra makra
lcDest=ALLT("&lcDest") && preventivn odstra makra

lcSourceN=lcSource+CHR(0) && piti nulov znak
lcDestN=lcDest+CHR(0) && piti nulov znak

IF RIGHT(lcSource,1)#'\' && pokud na konci nen lomtko
   lcSource=lcSource+'\' && pak ho pidej
ENDIF

IF RIGHT(lcDest,1)#'\' && pokud na konci nen lomtko
   lcDest=lcDest+'\' && pak ho pidej
ENDIF

IF ! INLIST(MKD(IIF(llLower,LOWER(lcDest),lcDest)),0,1) && pokud se vytvoen adrese nepoda
   RETURN .F. && pak se vra
ENDIF

lic=ADIR(laDir,lcSource+'*.*','AHRSD')
* pokud nco naet, pak to zpracuj
FOR lii=1 TO lic
    IF !(!laDir(lii,1)=="." AND !laDir(lii,1)=="..") && pokud se jedn o fiktivn adrese
       LOOP && pak sko na dal
    ENDIF
    IF RIGHT(laDir(lii,5),1)='D' && jde o adres a m se koprovat
       * pokud se m koprovat a nejde vytvoit
       IF llChild AND !CopyDirectory(lcSource+laDir(lii,1),lcDest+laDir(lii,1),llChild,llLower,lcPatern)
          RETURN .F. && pak se vra
       ENDIF
    ELSE
       * jinak jde o soubor
       IF LIKE(lcPatern,laDir(lii,1))
          lcFSource=lcSource+laDir(lii,1)+CHR(0) && zdroj
          lcFDest=lcDest+IIF(llLower,LOWER(laDir(lii,1)),laDir(lii,1))+CHR(0) && cl
           =SetLastError(0) && vynuluj chyby
          lierr=CopyFile(@lcFSource,@lcFDest,0) && kopie souboru
          IF lierr=0 && pokud dolo k chyb
             RETURN .F. && pak se vra
          ENDIF
      ENDIF
   ENDIF
NEXT
RETURN .T.
ENDPROC


******************************************************************************
* Zalo adres
******************************************************************************
PROCEDURE Mkd(lcPath)
* lcPath -  cesta
LOCAL lcPom,lcDriveRoot,lii,lcErr,liErr

lcPath=ALLT("&lcPath") && preventivn odstra makra

IF DIRECTORY(lcPath) && pokud dan cesta existuje
   RETURN 1 && pak se vra 
ENDIF

* patrn nejde o UNC nebo DOS cestu s diskem
IF !(LEFT(lcPath,2)="\\" OR SUBS(lcPath,2,1)=":")
   lcPom=ALLTRIM(SET("PATH")) && vra nastaven PATH a oe mezery
   IF LEFT(lcPath,1)="\" && jde o cestu z rootu, pak musm pidat jen disk (server+sdlen disk)
      IF LEFT(lcPom,2)="\\" && jde o UNC cestu
         lii=AT("\",lcPom,4) && najdi 4. oddlova
         IF lii>0 && pokud tam je
            lcPom=LEFT(lcPom,lii-1) && pak nati cestu bez oddlovae
         ENDIF
       ELSE
         * jinak jde o DOS cestu
         lcPom=LEFT(lcPom,2)
       ENDIF
   ELSE
      IF RIGHT(lcPom,1)#"\" && pokud na konci nen oddlova
         lcPom=lcPom+"\" && pak ho tam pidej
      ENDIF
   ENDIF
   lcPath=lcPom+lcPath && seti novou cestu
ENDIF

IF LEFT(lcPath,2)#"\\" && pokud to je DOS cesta
   lcDriveRoot=LEFT(lcPath,2)+"\" && vem si disk
   IF INLIST(GetDriveType(lcDriveRoot+CHR(0)),0,5) && Disk neexistuje, nebo je to CD-ROM
      RETURN 4  && vra se
   ENDIF
ENDIF

lcErr=ON("ERROR")
liErr=0
ON ERROR liErr=ERROR()
MD (lcPath) && zalo cestu
IF !EMPTY(lcErr)
   ON ERROR &lcErr.
ELSE
   ON ERROR
ENDIF

* pokud nen chyba, pak vra 0
* pstup zamtnut
* invalid path or name
RETURN IIF(liErr=0,0,;
      IIF(liErr=1705,2,;
      IIF(liErr=202,3,3)))  && vra pznak chyb
ENDPROC


******************************************************************************
* Smae obsah adrese
******************************************************************************
PROCEDURE RemoveDirectoryAll(lcSource,llDMain)
IF PCOUNT()<2 && pokud nen parametr
   llDMain=.T.
   IF PCOUNT()=0 && pokud nen parametr
      RETURN .F. && ErrFewArguments pak se vra
   ENDIF
ENDIF
LOCAL lcSourceN,lic,lierr,lierrx,lii,lcFSource
LOCAL ARRAY laDir(1)

lcSource=ALLT("&lcSource") && preventivn odstra makra
lcSourceN=lcSource+CHR(0) && piti nulov znak

IF RIGHT(lcSource,1)#'\' && pokud na konci nen lomtko
   lcSource=lcSource+'\' && pak ho pidej
ENDIF

lic=ADIR(laDir,lcSource+'*.*','AHRSD')
FOR lii=1 TO lic
    IF !(!laDir(lii,1)=="." AND !laDir(lii,1)=="..") && pokud se jedn o fiktivn adrese
       LOOP && pak sko na dal
    ENDIF
    IF RIGHT(laDir(lii,5),1)='D' && jde o adres a m se koprovat
       * pokud se m koprovat a nejde vytvoit
       IF ! RemoveDirectoryAll(lcSource+laDir(lii,1))
          RETURN .F. && ErrOther pak se vra
       ENDIF
    ELSE
       * jinak jde o soubor
       lcFSource=lcSource+laDir(lii,1)+CHR(0) && zdroj
        =SetLastError(0) && vynuluj chyby
       lierr=DeleteFile(@lcFSource) && vmaz souboru
       IF lierr=0 && pokud dolo k chyb
          RETURN .F. && ErrOther pak se vra
       ENDIF
   ENDIF
NEXT
IF llDMain
   =SetLastError(0) && vynuluj chyby
   liErr=RemoveDirectory(lcSourceN) && odstra adres
ELSE
   liErr=0
ENDIF



******************************************************************************
* Zkopruje obsah adrese do jinho adrese
******************************************************************************
PROCEDURE DeleteFiles(lcPath,llChild,lcPatern)
* lcPath   - Zdrojov adres
* llChild  - Pznak, prochzen podadres
* lcPatern - Patern pro vbr soubor

LOCAL lcFL,liErr
IF GetFileList(lcPath,lcPatern,llChild,@lcFL)>0
   SELE (lcFL)
   SCAN ALL
        * jinak jde o soubor
        liErr=DeleteFile((ALLT(XX000)+CHR(0))) && Smazat souboru
   ENDSCAN
ENDIF
IF USED(lcFL)
   USE IN (lcFL)
ENDIF

RETURN .T.
ENDPROC


******************************************************************************
* Zpakuje tabulky
******************************************************************************
PROCEDURE PackTables(lcPath,llChild)
* lcPath   - Zdrojov adres
* llChild  - Pznak, prochzen podadres
LOCAL lcAlias,lcFL
lcAlias=SYS(2015)
IF GetFileList(lcPath,"*.dbf",llChild,@lcFL)>0
   SELE (lcFL)
   SCAN ALL
        * jinak jde o soubor
        USE (ALLT(XX000)) EXCL IN 0 ALIAS (lcAlias)
        SELE (lcAlias)
        PACK MEMO
        PACK 
        REINDEX
        USE IN (lcAlias)
        SELE (lcFL)
   ENDSCAN
ENDIF
IF USED(lcFL)
   USE IN (lcFL)
ENDIF
ENDPROC

******************************************************************************
* Zbav VCX knihovny zdrojovch kd
******************************************************************************
PROCEDURE ClearVCXs(lcPath,llChild)
* lcPath   - Zdrojov adres
* llChild  - Pznak, prochzen podadres
LOCAL lcAlias,lcFL
lcAlias=SYS(2015)
IF GetFileList(lcPath,"*.vcx",llChild,@lcFL)>0
   SELE (lcFL)
   SCAN ALL
        * jinak jde o soubor
        USE (ALLT(XX000)) EXCL IN 0 ALIAS (lcAlias)
        SELE (lcAlias)
        REPL METHODS WITH "" ALL 
        PACK MEMO
        PACK 
        USE IN (lcAlias)
        SELE (lcFL)
   ENDSCAN
ENDIF
IF USED(lcFL)
   USE IN (lcFL)
ENDIF
ENDPROC

******************************************************************************
* Rekompilace VCX knihoven
******************************************************************************
PROCEDURE CompileVCXs(lcPath,llChild)
* lcPath   - Zdrojov adres
* llChild  - Pznak, prochzen podadres
LOCAL lcFL
IF GetFileList(lcPath,"*.vcx",llChild,@lcFL)>0
   SELE (lcFL)
   SCAN ALL
        * jinak jde o soubor
        COMPILE CLASSLIB (ALLT(XX000))
   ENDSCAN
ENDIF
IF USED(lcFL)
   USE IN (lcFL)
ENDIF
ENDPROC

******************************************************************************
* Rekompilace PRG soubor
******************************************************************************
PROCEDURE CompilePRGs(lcPath,llChild)
* lcPath   - Zdrojov adres
* llChild  - Pznak, prochzen podadres
LOCAL lcFL
IF GetFileList(lcPath,"*.prg",llChild,@lcFL)>0
   SELE (lcFL)
   SCAN ALL
        * jinak jde o soubor
        COMPILE (ALLT(XX000))
   ENDSCAN
ENDIF
IF USED(lcFL)
   USE IN (lcFL)
ENDIF
ENDPROC

*****************************************************
* return list of files from directory 
******************************************************
PROCEDURE GetFileList(lcDirs,lcPatern,llChild,lcAlias)
* lcDirs   - Seznam vstupnch adres
* lcPatern - Patern pro vbr soubor
* llChild  - Include subdirectories
*@lcAlias  - Alias vstupnho kurzoru

LOCAL lcAliasD,liCount,lii,liy,liz,lcDir,liRecno
LOCAL ARRAY laFiles(1)
lcAlias=SYS(2015)
lcAliasD=SYS(2015)

CREATE CURSOR (lcAlias) (XX000 C(254))
CREATE CURSOR (lcAliasD) (XX000 C(254))

lcDirs=ALLT(lcDirs)
lcDirs=lcDirs+IIF(RIGHT(lcDirs,1)=',','',',')
STORE OCCURS(',',lcDirs) TO liCount && Count of item

* Vytvo zkladn seznam adres
liy=1
FOR lii=1 TO liCount
    liz=AT(',',lcDirs,lii)
    lcDir=ALLT(SUBST(lcDirs,liy,liz-liy))
    lcDir=lcDir+IIF(RIGHT(lcDir,1)="\","","\")
 
    IF !EMPTY(lcDir)
       INSERT INTO (lcAliasD) (XX000) VALUES (lcDir)
    ENDIF
    liy=liz+1
NEXT

lcPatern=UPPER(lcPatern)

* Prochzej seznam adres a nati seznam soubor v nich
SELE (lcAliasD)
SCAN ALL
     lcDir=ALLT(XX000)
     liRecno=RECNO() && Zapamatuj si vtu
     liCount=ADIR(laFiles,lcDir+"*.*","ARSHD")

     FOR lii=1 TO liCount
         IF laFiles(lii,1)="." OR laFiles(lii,1)=".."
            LOOP
         ENDIF

         IF AT("D",laFiles(lii,5))>0 && Adres
            IF llChild
               INSERT INTO (lcAliasD) (XX000) VALUES (lcDir+laFiles(lii,1)+"\")
            ENDIF
         ELSE
            IF EMPTY(lcPatern) OR LIKE(lcPatern,UPPER(laFiles(lii,1)))
               * Jinak je to soubor
               INSERT INTO (lcAlias) (XX000) VALUES (lcDir+laFiles(lii,1))
            ENDIF
         ENDIF
     NEXT

     SELE (lcAliasD)
     GO (liRecno) && Sko na pvodn vtu
ENDSCAN
USE IN (lcAliasD)
RELE ARRAY laFiles(1)
RETURN RECC(lcAlias)


*****************************************************
* Write Text
******************************************************
PROCEDURE WriteText(lcText,liIndex)
@ 4+liIndex,0 SAY PADR(lcText,254)


